<?php
/**
 * Plugin Name: Umva Pay Gateway for WooCommerce
 * Description: Custom payment gateway for WooCommerce integrating with Umva Pay. <a href="https://pay.umva.us" target="_blank">Visit Site</a>
 * Version: 1.8
 * Author: Mizero Samuel
 * License: GPL2
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Include the gateway class
function umva_pay_gateway_init() {
    if (!class_exists('WC_Payment_Gateway')) return;

    class WC_Gateway_Umva_Pay extends WC_Payment_Gateway {

        public function __construct() {
            $this->id = 'umva_pay';
            $this->icon = ''; // URL of the icon that will be displayed on the checkout page
            $this->has_fields = true;
            $this->method_title = 'Umva Pay';
            $this->method_description = 'Allows payments with Umva Pay.';

            // Load the settings
            $this->init_form_fields();
            $this->init_settings();

            // Turn these settings into variables we can use
            foreach ($this->settings as $setting_key => $value) {
                $this->$setting_key = $value;
            }

            // Actions
            add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
            add_action('woocommerce_api_wc_gateway_umva_pay', array($this, 'check_ipn_response'));
            add_action('woocommerce_checkout_after_order_review', array($this, 'add_currency_field'));

            // Save settings
            if (is_admin()) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
            }
        }

        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => 'Enable/Disable',
                    'type' => 'checkbox',
                    'label' => 'Enable Umva Pay Payment Gateway',
                    'default' => 'no',
                ),
                'title' => array(
                    'title' => 'Title',
                    'type' => 'text',
                    'description' => 'This controls the title which the user sees during checkout.',
                    'default' => 'Umva Pay',
                ),
                'description' => array(
                    'title' => 'Description',
                    'type' => 'textarea',
                    'description' => 'This controls the description which the user sees during checkout.',
                    'default' => 'Pay securely using Umva Pay.',
                ),
                'public_key' => array(
                    'title' => 'Public API Key',
                    'type' => 'text',
                    'description' => 'Your Umva Pay Public API Key.',
                    'default' => '*******************',
                ),
                'secret_key' => array(
                    'title' => 'Secret API Key',
                    'type' => 'text',
                    'description' => 'Your Umva Pay Secret API Key.',
                    'default' => '*********************',
                ),
                'test_mode' => array(
                    'title' => 'Test Mode',
                    'type' => 'checkbox',
                    'label' => 'Enable Test Mode',
                    'default' => 'no',
                ),
                'ipn_url' => array(
                    'title' => 'IPN URL',
                    'type' => 'text',
                    'description' => 'URL to receive payment notifications.',
                    'default' => 'https://pay.umva.us',
                ),
                'success_url' => array(
                    'title' => 'Success URL',
                    'type' => 'text',
                    'description' => 'URL to redirect on payment success.',
                    'default' => site_url(),
                ),
                'cancel_url' => array(
                    'title' => 'Cancel URL',
                    'type' => 'text',
                    'description' => 'URL to redirect on payment cancellation.',
                    'default' => site_url(),
                ),
                'supported_currencies' => array(
                    'title' => 'Supported Currencies',
                    'type' => 'multiselect',
                    'description' => 'Select currencies you want to accept payments in.',
                    'default' => '',
                    'options' => array(
                        'BTC' => 'BTC',
                        'ETH' => 'ETH',
                        'USDT.TRC20' => 'USDT.TRC20',
                        'LTC' => 'LTC',
                        'DOGE' => 'DOGE',
                        'BNB.BSC' => 'BNB.BSC',
                        'EUR' => 'EUR',
                        'GBP' => 'GBP',
                        'GHS' => 'GHS',
                        'KES' => 'KES',
                        'UGX' => 'UGX',
                        'RWF' => 'RWF',
                        'ZMW' => 'ZMW',
                        'TZS' => 'TZS',
                        'XAF' => 'XAF',
                        'XOF' => 'XOF',
                        'NGN' => 'NGN',
                        'MWK' => 'MWK',
                        'USD' => 'USD',
                    ),
                ),
                'conversion_rates' => array(
    'title' => 'Conversion Rates',
    'type' => 'textarea',
    'description' => 'Enter conversion rates for supported currencies based on the base currency.',
    'default' => "BTC:0.000026\nETH:0.0004\nUSDT.TRC20:1.0\nLTC:0.004\nDOGE:15.0\nBNB.BSC:0.002\nEUR:0.85\nGBP:0.75\nGHS:12.0\nKES:110.0\nUGX:3800.0\nRWF:1200.0\nZMW:20.0\nTZS:2300.0\nXAF:550.0\nXOF:550.0\nNGN:500.0\nMWK:800.0\nUSD:1.0",
    'placeholder' => 'Currency:Rate\nUSD:1.0\nEUR:0.9',
),

                
            );
        }

        public function process_payment($order_id) {
            $order = wc_get_order($order_id);

            // Get the selected currency from the order meta
            $currency = isset($_POST['selected_currency']) ? sanitize_text_field($_POST['selected_currency']) : strtoupper(get_woocommerce_currency());

            // Ensure the currency is supported
            $supported_currencies = array_map('trim', explode(',', $this->supported_currencies));
            if (!in_array($currency, $supported_currencies)) {
                wc_add_notice('The selected currency is not supported by Umva Pay.', 'error');
                return;
            }

            // Get conversion rate
            $conversion_rates = array_filter(array_map('trim', explode("\n", $this->conversion_rates)), function($rate) {
                return strpos($rate, ':') !== false;
            });
            $conversion_rate = 1.0; // Default conversion rate if not found
            foreach ($conversion_rates as $rate) {
                list($curr, $rate_value) = explode(':', $rate);
                if (strtoupper($curr) === $currency) {
                    $conversion_rate = floatval($rate_value);
                    break;
                }
            }

            $amount = $order->get_total() * $conversion_rate;

            $parameters = array(
                'identifier' => $order_id,
                'currency' => $currency,
                'amount' => $amount,
                'details' => 'Order #' . $order_id,
                'ipn_url' => $this->ipn_url,
                'success_url' => $this->get_return_url($order),
                'cancel_url' => $this->get_cancel_url(),
                'public_key' => $this->public_key,
                'site_name' => get_bloginfo('name'),
                'site_logo' => '',
                'checkout_theme' => 'light',
                'customer' => array(
                    'first_name' => $order->get_billing_first_name(),
                    'last_name' => $order->get_billing_last_name(),
                    'email' => $order->get_billing_email(),
                    'mobile' => $order->get_billing_phone(),
                ),
            );

            $parameters = http_build_query($parameters);
            $url = 'https://pay.umva.us/payment/initiate'; // Default to live URL

            $response = wp_remote_post($url, array(
                'body' => $parameters,
                'timeout' => 45,
                'sslverify' => true,
            ));

            if (is_wp_error($response)) {
                wc_add_notice('Payment error: ' . $response->get_error_message(), 'error');
                return;
            }

            $response_body = wp_remote_retrieve_body($response);
            $response_data = json_decode($response_body, true);

            if ($response_data['status'] === 'success') {
                return array(
                    'result' => 'success',
                    'redirect' => $response_data['redirect_url'],
                );
            } else {
                wc_add_notice('Payment error: ' . $response_data['message'][0], 'error');
                return;
            }
        }

        public function receipt_page($order) {
            echo '<p>Thank you for your order. Redirecting to Umva Pay for payment...</p>';
            echo '<script type="text/javascript">
                window.location.href = "' . esc_url($this->get_return_url(wc_get_order($order))) . '";
            </script>';
        }

        public function check_ipn_response() {
            if (!isset($_POST['status'], $_POST['signature'], $_POST['identifier'], $_POST['data'])) {
                wp_redirect($this->get_cancel_url());
                exit;
            }

            $status = sanitize_text_field($_POST['status']);
            $signature = sanitize_text_field($_POST['signature']);
            $identifier = sanitize_text_field($_POST['identifier']);
            $data = json_decode(stripslashes($_POST['data']), true);

            $order_id = wc_get_order_id_by_order_key($identifier);

            if (!$order_id) {
                wp_redirect($this->get_cancel_url());
                exit;
            }

            $order = wc_get_order($order_id);

            if ($status === 'success') {
                $order->payment_complete();
                $order->add_order_note('Payment received via Umva Pay.');
                wp_redirect($this->get_return_url($order));
            } else {
                $order->update_status('failed', 'Payment failed via Umva Pay.');
                wp_redirect($this->get_cancel_url());
            }
            exit;
        }

        public function add_currency_field() {
            $supported_currencies = array(
                'BTC' => 'BTC',
                'ETH' => 'ETH',
                'USDT.TRC20' => 'USDT.TRC20',
                'LTC' => 'LTC',
                'DOGE' => 'DOGE',
                'BNB.BSC' => 'BNB.BSC',
                'EUR' => 'EUR',
                'GBP' => 'GBP',
                'GHS' => 'GHS',
                'KES' => 'KES',
                'UGX' => 'UGX',
                'RWF' => 'RWF',
                'ZMW' => 'ZMW',
                'TZS' => 'TZS',
                'XAF' => 'XAF',
                'XOF' => 'XOF',
                'NGN' => 'NGN',
                'MWK' => 'MWK',
                'USD' => 'USD'
            );
        
            echo '<div id="umva-pay-currency">
                <h3>Select Payment Currency</h3>
                <select name="selected_currency" id="selected_currency">';
        
            foreach ($supported_currencies as $code => $name) {
                echo '<option value="' . esc_attr($code) . '">' . esc_html($name) . '</option>';
            }
        
            echo '</select></div>';
        }
        
    }
}

add_action('plugins_loaded', 'umva_pay_gateway_init');


function add_umva_pay_gateway($methods) {
    $methods[] = 'WC_Gateway_Umva_Pay';
    return $methods;
}

add_filter('woocommerce_payment_gateways', 'add_umva_pay_gateway');
